
public class HoffmanTree
{
	private BinNode<CharFreq> root;
	private String[] codes;
	
	public HoffmanTree(CharFreq[] chars)
	{
		this.codes=new String[256];
		PriorityQueue q=new PriorityQueue();
		for (int i=0; i<chars.length; i++)
		{
			BinNode<CharFreq> temp=new BinNode<CharFreq> (chars[i]);
			q.insert(temp);
		}
		this.root=buildTree (q);
		codeTree();
	}
	
	private BinNode<CharFreq> buildTree (PriorityQueue q)
	{
		BinNode<CharFreq> first=q.remove();
		BinNode<CharFreq> second;
		if (!q.isEmpty())
		{
			second=q.remove();
			int freq=first.getValue().getFreq()+second.getValue().getFreq();
			BinNode<CharFreq> current=new BinNode<CharFreq> (new CharFreq('$',freq),first,second);
			q.insert(current);
			return buildTree (q);
		}
		return first;
	}
	
	
	public BinNode<CharFreq> getRoot()
	{
		return this.root;
	}
	
	public String[]  getCodes()
	{
		return this.codes;
	}
	
	private void codeTree()
	{
		buildHoffmanCode(this.root, new char [Utilities.treeHeight(this.root)],0);
	}	
	
	private void buildHoffmanCode(BinNode<CharFreq> root, char arr[], int top)
	{
	    // Assign 0 to left edge and recur
	    if (root.hasLeft())
	    {
	        arr[top] = '0';
	        buildHoffmanCode(root.getLeft(), arr, top + 1);
	    }
	 
	    // Assign 1 to right edge and recur
	    if (root.hasRight())
	    {
	        arr[top] = '1';
	        buildHoffmanCode(root.getRight(), arr, top + 1);
	    }
	 
	    // If this is a leaf node, then it contains one of the input
	    // characters, print the character and its code from arr[]
	    if (!root.hasLeft() && !root.hasRight())
	    {
	       this.codes[root.getValue().getTav()]=Utilities.buildStringCode(arr,top);
	    }
	}
	
	public String codeMessage(String str)
	{
		String coded="";
		for (int i=0; i<str.length();i++)
			coded=coded+this.codes[str.charAt(i)];
		return coded;
	}
	
	public String decodeMessage (String binStr)
	{
		String result="";
		int index=0;
		BinNode<CharFreq> t=this.root;
		while (index!=binStr.length())
		{
			//System.out.println(index+","+t.getValue().getTav());
			if (!t.hasLeft() && !t.hasRight())
			{
				result=result+t.getValue().getTav();
				t=this.root;
			}
			else
			{
				if (binStr.charAt(index)=='0')
					t=t.getLeft();
				else
					t=t.getRight();
				index++;
			}
		}
		return result+t.getValue().getTav();
	}
	
	public void printCodes()
	{
		for (int i=0; i<256; i++)
			if (this.codes[i]!=null)
				System.out.println((char)i+","+this.codes[i]);
	}

}
